/***********************************************************
 * $Id$
 *
 * FancySAX
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSAXCONTENTHANDLER_H_
#define FANCYSAXCONTENTHANDLER_H_

#include <fancysax.h>
#include <fancyobject.h>

namespace Fancy {

namespace SAX {

/**
 * A SAX2 content handler interface and a support class which does debug logging in all functions.
 */
class FANCYSAX_API ContentHandler: public virtual Fancy::Object {

public:
    typedef Fancy::ObjPtr<ContentHandler> ptr_t;

    /**
     * Return a new default instance, which does debug logging and nothing else.
     */
    static ptr_t newDefaultInstance();

    /**
     * Called when the document start being processed.
     */
    virtual void startDocument();

    /**
     * Called when the document end has been detected.
     */
    virtual void endDocument();

    /**
     * SAX2 callback when an element start has been detected by the parser.
     * It provides the namespace informations for the element, as well as
     * the new namespace declarations on the element.
     *
     * @param localname  the local name of the element
     * @param prefix  the element namespace prefix if available
     * @param URI  the element namespace name if available
     * @param nb_namespaces  number of namespace definitions on that node
     * @param namespaces  pointer to the array of prefix/URI pairs namespace definitions
     * @param nb_attributes  the number of attributes on that node
     * @param nb_defaulted  the number of defaulted attributes. The defaulted
     *                  ones are at the end of the array @c attributes.
     * @param attributes  pointer to the array of (localname/prefix/URI/value/end)
     *                    attribute values. This array has a size of @c nb_attributes.
     */
    virtual void startElementNs (const char * localname,
                                 const char * prefix,
                                 const char * URI,
                                 int nb_namespaces,
                                 const char ** namespaces,
                                 int nb_attributes,
                                 int nb_defaulted,
                                 const Attribute* attributes);

    /**
     * SAX2 callback when an element end has been detected by the parser.
     * It provides the namespace informations for the element.
     *
     * @param localname  the local name of the element
     * @param prefix  the element namespace prefix if available
     * @param URI  the element namespace name if available
     */
    virtual void endElementNs (const char * localname,
                               const char * prefix,
                               const char * URI);

    /**
     * Called when an entity reference is detected.
     *
     * @param name  The entity name
     */
    virtual void reference (const char *name);

    /**
     * Receiving some chars from the parser.
     *
     * @param ch  an UTF-8 encoded string
     * @param len the number of UTF-8 characters
     */
    virtual void characters (const char *ch,
                             int len);

    /**
     * Receiving some ignorable whitespaces from the parser.
     *
     * @param ch  a xmlChar string
     * @param len the number of xmlChar
     */
    virtual void ignorableWhitespace (const char *ch,
                                      int len);

    /**
     * A processing instruction has been parsed.
     *
     * @param target  the target name
     * @param data the PI data's
     */
    virtual void processingInstruction (const char *target,
                                        const char *data);

    /**
     * A comment has been parsed.
     *
     * @param value  the comment content
     */
    virtual void comment (const char *value);

    /**
     * Called when a pcdata block has been parsed.
     *
     * @param value  The pcdata content
     * @param len  the block length
     */
    virtual void cdataBlock (const char *value,
                             int len);

protected:
	ContentHandler();
	virtual ~ContentHandler();
};

}

}

#endif /* FANCYSAXCONTENTHANDLER_H_ */
