/***********************************************************
 * $Id$
 *
 * FancySAX
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSAXREADER_H_
#define FANCYSAXREADER_H_

#include <fancyobject.h>
#include <fancysax.h>

namespace Fancy {

class Channel;
class String;
class Integer;
class Long;
class Double;
class UtcTimestamp;

namespace SAX {

class SchemaValidator;

/**
 * A SAX stream reader interface.
 */
class FANCYSAX_API Reader: public virtual Fancy::Object {

public:
    typedef Fancy::ObjPtr<Reader> ptr_t;

    enum NodeType {
        XML_READER_TYPE_NONE = 0,
        XML_READER_TYPE_ELEMENT = 1,
        XML_READER_TYPE_ATTRIBUTE = 2,
        XML_READER_TYPE_TEXT = 3,
        XML_READER_TYPE_CDATA = 4,
        XML_READER_TYPE_ENTITY_REFERENCE = 5,
        XML_READER_TYPE_ENTITY = 6,
        XML_READER_TYPE_PROCESSING_INSTRUCTION = 7,
        XML_READER_TYPE_COMMENT = 8,
        XML_READER_TYPE_DOCUMENT = 9,
        XML_READER_TYPE_DOCUMENT_TYPE = 10,
        XML_READER_TYPE_DOCUMENT_FRAGMENT = 11,
        XML_READER_TYPE_NOTATION = 12,
        XML_READER_TYPE_WHITESPACE = 13,
        XML_READER_TYPE_SIGNIFICANT_WHITESPACE = 14,
        XML_READER_TYPE_END_ELEMENT = 15,
        XML_READER_TYPE_END_ENTITY = 16,
        XML_READER_TYPE_XML_DECLARATION = 17
    };

    /**
     * @param channel the channel to read from
     * @param url The URL of the resource used for logging and error messages.
     * @return A new reader instance parsing the given channel.
     */
    static ptr_t newInstance(const Fancy::ObjPtr<Channel> &channel, const std::string &url);

    /**
     * Set up a reader which evaluates an XML schema.
     * After parsing the whole document, you might call
     * @c SchemaValidtor::isSchemaValid() to find out, whether the parsed
     * document was valid.
     *
     * @param channel the channel to read from
     * @param url The URL of the resource used for logging and error messages.
     * @param schemaValidator A schema validator, which evaluates the schema
     *                        conformance as the stream is parsed.
     * @return A new reader instance parsing the given channel.
     */
    static ptr_t newInstance(
            const Fancy::ObjPtr<Channel> &channel,
            const std::string &url,
            const Fancy::ObjPtr<SchemaValidator> &schemaValidator);

    /**
     * Move to the next tag and return false, if the stream ended.
     */
    virtual bool nextTag() = 0;

    /**
     * Check the tag name for a given node name consisting
     * of a namespace prefix, a colon and a local name.
     *
     * A faster version of this->getNodeName() == nodeName
     */
    virtual bool checkTag(const char *nodeName) const = 0;

    /**
     * Check the tag name for a given local name without any
     * namespace information.
     *
     * A faster version of this->getLocaleName() == localName
     */
    virtual bool checkTagLocalName(const char *localName) const = 0;

    /**
     * Check the tag name for a given node name consisting
     * of a namespace prefix, a colon and a local name.
     *
     * A faster version of
     *
     *   this->getLocalName() == localName && this->getNamespaceURI() == namespaceURI
     */
    virtual bool checkTagNs(const char *localName, const char *namespaceURI) const = 0;

    /**
     * The name of the current node, which may consist of a prefix and a local
     * name for namespaces with prefixes.
     */
    virtual std::string getNodeName() const = 0;

    /**
     * The namespace prefix.
     */
    virtual std::string getPrefix() const = 0;

    /**
     * The namespace URI of the current node.
     */
    virtual std::string getNamespaceURI() const = 0;

    /**
     * The local name of the current node.
     */
    virtual std::string getLocalName() const = 0;

    /**
     * The depth of the current node.
     */
    virtual int getDepth() const = 0;

    /**
     * The node type, see NodeType enum values.
     */
    virtual int getNodeType() const = 0;

    /**
     * Return, whether the current node is empty.
     */
    virtual bool isEmpty() const = 0;

    /**
     * Return, whether the current node may have a value.
     */
    virtual bool hasValue() const = 0;

    /**
     * Get the value of a given attribute or null, if the attribute has not been specified.
     */
    virtual Fancy::ObjPtr<Fancy::String> getAttributeObjValue(const char *name) const = 0;

    /**
     * Get the value of a given attribute or an empty string, if the attribute has not been specified.
     */
    virtual std::string getAttributeValue(const char *name) const = 0;

    /**
     * Get the integer object value of a given attribute or null, if the attribute has not been specified.
     */
    virtual Fancy::ObjPtr<Fancy::Integer> getAttributeIntObjValue(const char *name) const = 0;

    /**
     * Get the integer value of a given attribute or 0, if the attribute has not been specified.
     */
    virtual int getAttributeIntValue(const char *name) const = 0;

    /**
     * Get the long object value of a given attribute or null, if the attribute has not been specified.
     */
    virtual Fancy::ObjPtr<Fancy::Long> getAttributeLongObjValue(const char *name) const = 0;

    /**
     * Get the long value of a given attribute or 0, if the attribute has not been specified.
     */
    virtual int64_t getAttributeLongValue(const char *name) const = 0;

    /**
     * Get the double object value of a given attribute or null, if the attribute has not been specified.
     */
    virtual Fancy::ObjPtr<Fancy::Double> getAttributeDoubleObjValue(const char *name) const = 0;

    /**
     * Get the double value of a given attribute or 0, if the attribute has not been specified.
     */
    virtual double getAttributeDoubleValue(const char *name) const = 0;

    /**
     * Get the value of a given attribute by namespace URI and local name or null,
     * if the attribute has not been specified.
     */
    virtual Fancy::ObjPtr<Fancy::String> getAttributeObjValueNs(const char *localName, const char *namespaceURI) const = 0;

    /**
     * Get the value of a given attribute or an empty string, if the attribute has not been specified.
     */
    virtual std::string getAttributeValueNs(const char *localName, const char *namespaceURI) const = 0;

    /**
     * Get the integer object value of a given attribute or null, if the attribute has not been specified.
     */
    virtual Fancy::ObjPtr<Fancy::Integer> getAttributeIntObjValueNs(const char *localName, const char *namespaceURI) const = 0;

    /**
     * Get the integer value of a given attribute or 0, if the attribute has not been specified.
     */
    virtual int getAttributeIntValueNs(const char *localName, const char *namespaceURI) const = 0;

    /**
     * Get the long object value of a given attribute or null, if the attribute has not been specified.
     */
    virtual Fancy::ObjPtr<Fancy::Long> getAttributeLongObjValueNs(const char *localName, const char *namespaceURI) const = 0;

    /**
     * Get the long value of a given attribute or 0, if the attribute has not been specified.
     */
    virtual int64_t getAttributeLongValueNs(const char *localName, const char *namespaceURI) const = 0;

    /**
     * Get the double object value of a given attribute or null, if the attribute has not been specified.
     */
    virtual Fancy::ObjPtr<Fancy::Double> getAttributeDoubleObjValueNs(const char *localName, const char *namespaceURI) const = 0;

    /**
     * Get the double value of a given attribute or 0, if the attribute has not been specified.
     */
    virtual double getAttributeDoubleValueNs(const char *localName, const char *namespaceURI) const = 0;

    /**
     * The inner XML of the current node.
     */
    virtual std::string readInnerXml() = 0;

    /**
     * The outer XML of the current node, that is.
     */
    virtual std::string readOuterXml() = 0;

    /**
     * The current node's text value or an empty string, if the node is empty.
     */
    virtual std::string getNodeValue() const = 0;

    /**
     * The current node's text value or null, if the node is empty.
     */
    virtual Fancy::ObjPtr<Fancy::String> getNodeObjValue() const = 0;

    /**
     * Get the integer object value of the current node or null, if the node is empty.
     */
    virtual Fancy::ObjPtr<Fancy::Integer> getNodeIntObjValue() const = 0;

    /**
     * Get the integer value of the current node or 0, if the node is empty.
     */
    virtual int getNodeIntValue() const = 0;

    /**
     * Get the long object value of the current node or null, if the node is empty.
     */
    virtual Fancy::ObjPtr<Fancy::Long> getNodeLongObjValue() const = 0;

    /**
     * Get the long value of the current node or 0, if the node is empty.
     */
    virtual int64_t getNodeLongValue() const = 0;

    /**
     * Get the double object value of the current node or null, if the node is empty.
     */
    virtual Fancy::ObjPtr<Fancy::Double> getNodeDoubleObjValue() const = 0;

    /**
     * Get the double value of the current node or 0, if the node is empty.
     */
    virtual double getNodeDoubleValue() const = 0;

    // new functions as of verion 0.6

    /**
     * Get the ISO8601 timestamp value of the given attribute or null, if the attribute does not exist.
     */
    virtual Fancy::ObjPtr<Fancy::UtcTimestamp> getAttributeTimestampValue(const char *name) const = 0;

    /**
     * Get the ISO8601 timestamp value of the given attribute or null, if the attribute does not exist.
     */
    virtual Fancy::ObjPtr<Fancy::UtcTimestamp> getAttributeTimestampValueNs(const char *localName, const char *namespaceURI) const = 0;

    /**
     * Get the ISO8601 timestamp value of the current node or null, if the node is empty.
     */
    virtual Fancy::ObjPtr<Fancy::UtcTimestamp> getNodeTimestampValue() const = 0;

protected:
    Reader();
    virtual ~Reader();
};

}

}

#endif /* FANCYSAXREADER_H_ */
